package cli

import (
	"klog/app"
	"klog/parser/json"
	"klog/parser/parsing"
)

type Json struct {
	FilterArgs
	SortArgs
	InputFilesArgs
	Pretty bool `name:"pretty" help:"Pretty-print output"`
}

func (opt *Json) Run(ctx app.Context) error {
	records, err := ctx.RetrieveRecords(opt.File...)
	if err != nil {
		parserErrs, isParserErr := err.(parsing.Errors)
		if isParserErr {
			ctx.Print(json.ToJson(nil, parserErrs, opt.Pretty) + "\n")
			return nil
		}
		return err
	}
	records = opt.filter(ctx.Now(), records)
	records = opt.sort(records)
	ctx.Print(json.ToJson(records, nil, opt.Pretty) + "\n")
	return nil
}
